/*
 * Created on Jan 18, 2005
 */
package proxyconfig;

import java.util.*;

import proxyconfig.i3.I3AddressBook;
import proxyconfig.i3.NATServerWizardDialog;
import proxyconfig.mynames.*;
import proxyconfig.ocd.OCD;
import proxyconfig.ocd.OCDPanel;
import proxyconfig.outboundconnections.*;
import proxyconfig.rsptunnelocd.SecureConnectionWizardDialog;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.*;

import javax.swing.*;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.io.*;
import java.net.URL;
import java.util.ListIterator;

/**
 * This is the main window of the proxy configurator.
 * 
 * @author danjo
 */
public class ProxyConfig extends JFrame implements ActionListener {

    public static final Color INPUT_BOX_COLOR = new Color(240, 255, 240);

    public static final Color HEADING_COLOR = new Color(230, 230, 250);

    public static final Color HELP_BUTTON_COLOR = new Color(150, 205, 205);

    public static Color BACKGROUND_COLOR;

    public static Color WHITE_COLOR = Color.WHITE;

    public static final String FIRST_TIME_CHECK_FILE = ".ProxyConfig.firsttime";

    public JFileChooser fc = new JFileChooser();

    public File proxyConfigFile = new File("config_files/proxy-config.xml");

    public ProxyConfigData proxyConfigData;

    public boolean wizardFlag = false;

    public boolean firstTimeFlag = false;

    public boolean startProxyFlag = false;

    OutboundPanel outboundPanel;

    MyNamePanel myNamePanel;

    JButton nextTabBT, prevTabBT;

    WelcomePane welcomePane;

    OCDPanel ocdPanel;

    AdvancedOptionsPanel advancedOptionsPanel;

    JTabbedPane tabPane;

    public GUIConfigReader guiConfigReader;

    public InputChangeListener changeListener = new InputChangeListener();

    public final String windowTitle = "Proxy Configuration Wizard";

    public static void main(String[] args) {

        new ProxyConfig(args);

    }

    ProxyConfig(String[] args) {

        if (args.length > 0 && args[0].trim().toLowerCase().equals("show_windows_post_install")) {
	    showWindowsPostInstallMsg();
	    System.exit (0);
	}


        BACKGROUND_COLOR = (Color) UIManager.get("Panel.background");

        if (args.length >= 1) {
            if (args[0].trim().toLowerCase().equals("wizard")) {
                wizardFlag = true;
            } else {
                wizardFlag = false;
            }
        }

        // Check if this is the first time we are running ProxyConfig
        firstTimeFlag = checkFirstTime();

        // Load the GUI configurations from gui.xml
        guiConfigReader = new GUIConfigReader(this);

        setTitle(windowTitle);
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

        if (proxyConfigFile.exists()) {
            setTitle(windowTitle + " [" + proxyConfigFile.getPath() + "]");
        }

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent arg0) {
                exitConfig();
                ;
            }
        });

        int xInsets = 130;
        int yInsets = 90;

        //Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension screenSize = new Dimension(1024, 768);

        setResizable(false);

        setBounds(xInsets, yInsets, screenSize.width - xInsets * 2,
                screenSize.height - yInsets * 2);

        if (proxyConfigFile.exists()) {
            proxyConfigData = ProxyConfigFileReader.loadFromFile(
                    proxyConfigFile, guiConfigReader);
            proxyConfigData.proxyConfig = this;
        } else {
            System.out.println("Unable to find " + proxyConfigFile
                    + ". Unable to continue.\n");
            System.exit(-1);
            //TODO fix this
        }

        // Load myNames
        File myNameFile = new File(proxyConfigData.myNameConfigFileName);
        if (myNameFile.exists()) {

            try {

                new MyNameConfigFileReader(myNameFile,
                        proxyConfigData.myNameList);
                // Go through each name and fill in the ocd pointer based on the
                // ocd id.
                Iterator itr = proxyConfigData.myNameList.listIterator();
                while (itr.hasNext()) {

                    MyName currName = (MyName) itr.next();
                    currName.ocd = proxyConfigData
                            .getOverlayById(currName.ocdId);

                }
            } catch (SAXParseException spe) {
                System.out.println(spe + "  line:" + spe.getLineNumber()
                        + " Message: " + spe.getMessage() + " column: "
                        + spe.getColumnNumber() + " uri: " + spe.getSystemId());
            } catch (SAXException sxe) {
                System.out.println(sxe);
            }

        } else {
            JOptionPane.showMessageDialog(null, "MyName configuration file "
                    + myNameFile + " missing.", "Missing File",
                    JOptionPane.ERROR_MESSAGE);
        }

        // Load outbound connection preferences
        File outboundFile = new File(proxyConfigData.outboundConfigFileName);
        if (outboundFile.exists()) {

            try {

                new OutboundConfigFileReader(outboundFile,
                        proxyConfigData.outboundList);

                // Go through outbound record and fill in the ocd pointer based
                // on the ocd id.
                Iterator itr = proxyConfigData.outboundList.listIterator();
                while (itr.hasNext()) {

                    OutboundRecord currRec = (OutboundRecord) itr.next();
                    ListIterator hopItr = currRec.hopList.listIterator();
                    while (hopItr.hasNext()) {
                        HopInfo currHop = (HopInfo) hopItr.next();
                        currHop.ocd = proxyConfigData
                                .getOverlayById(currHop.ocdId);
                    }
                }

            } catch (SAXParseException spe) {
                System.out.println(spe + "  line:" + spe.getLineNumber()
                        + " Message: " + spe.getMessage() + " column: "
                        + spe.getColumnNumber() + " uri: " + spe.getSystemId());
            } catch (SAXException sxe) {
                System.out.println(sxe);
            }

        } else {
            JOptionPane.showMessageDialog(null, "MyName configuration file "
                    + myNameFile + " missing.", "Missing File",
                    JOptionPane.ERROR_MESSAGE);
        }

        initGUI();

        if (args.length > 0 && args[0].trim().toLowerCase().equals("secureconnection")) {
            SecureConnectionWizardDialog secureConnectionWizard = new SecureConnectionWizardDialog (this);
            secureConnectionWizard.show();
            System.exit (0);
        }

        if (args.length > 0 && args[0].trim().toLowerCase().equals("natserver")) {
	    NATServerWizardDialog natServerWizardDialog = new NATServerWizardDialog(this);
            natServerWizardDialog.show();
	    if (natServerWizardDialog.isRestartRequired()) {
	    	JOptionPane.showMessageDialog (this, "OCALA proxy must be restarted for any changes to take effect.", "Restart Necessary", JOptionPane.INFORMATION_MESSAGE);
	    }
            System.exit (0);
        }

        if (firstTimeFlag) {

            WelcomeWizardDialog welcomeWizardDialog = new WelcomeWizardDialog(
                    this);
            welcomeWizardDialog.show();

        } else {
            show();
        }
    }

    static Object panelBackgroundColor;

    static Object optionPaneBackgroundColor;


    private void showWindowsPostInstallMsg() {

        URL helpURL = null;
        try {
            JEditorPane mainEditorPane = new JEditorPane ();
           
            helpURL = getClass().getResource("/helpsnippets/WindowsSetup_PostInstallMsg.html");
            mainEditorPane.setPage (helpURL);
            mainEditorPane.setEditable(false);
            //mainEditorPane.addHyperlinkListener(this);
            
            JScrollPane scrollPane = new JScrollPane (mainEditorPane);
	    	scrollPane.setPreferredSize (new Dimension (400,550));
	    	JOptionPane.showMessageDialog ( null, 
					scrollPane,
					"OCALA starting up",
					JOptionPane.INFORMATION_MESSAGE
				);
        } catch (IOException ioe) {
	    	System.out.println ( "Unable to display post install information " + helpURL + " : " + ioe);
	    	ioe.printStackTrace();
	}
    }

    public static void changeOptionPaneBackground(Color c) {

        panelBackgroundColor = UIManager.get("Panel.background");
        optionPaneBackgroundColor = UIManager.get("OptionPane.background");

        //UIManager.put("Panel.background",c);
        //UIManager.put("OptionPane.background",c);

    }

    public static void restoreOptionPaneBackground() {
        //UIManager.put("Panel.background",panelBackgroundColor);
        //UIManager.put("OptionPane.background",optionPaneBackgroundColor);

    }

    private void initGUI() {

        Container contents = getContentPane();
        contents.setLayout(new BoxLayout(contents, BoxLayout.X_AXIS));

        //Add general left hand graphic.
        String lhsPicFile = "campanile.jpg";
        URL lhsPicURL = getClass().getResource("/" + lhsPicFile);
        contents.add(new JLabel(new ImageIcon(Toolkit.getDefaultToolkit()
                .getImage(lhsPicURL))));

        contents.add(Box.createRigidArea(new Dimension(5, 5)));

        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
        contents.add(mainPanel);

        tabPane = new JTabbedPane();
        mainPanel.add(tabPane);

        //welcomePane = new WelcomePane(this);
        //tabPane.addTab("Welcome", welcomePane);

        ocdPanel = new OCDPanel(this);
        tabPane.addTab("Overlays", ocdPanel);

        myNamePanel = new MyNamePanel(this);
        tabPane.addTab("My Names", myNamePanel);

        outboundPanel = new OutboundPanel(this);
        tabPane.addTab("Outbound Connections", outboundPanel);

        advancedOptionsPanel = new AdvancedOptionsPanel(this);
        tabPane.addTab("Advanced Options", advancedOptionsPanel);

        /*
         * JPanel buttonPanel = new JPanel(); buttonPanel.setLayout(new
         * BoxLayout(buttonPanel, BoxLayout.X_AXIS));
         * mainPanel.add(buttonPanel);
         * 
         * prevTabBT = new JButton(" < < < < < <");
         * prevTabBT.setBackground(Color.BLUE);
         * prevTabBT.addActionListener(this);
         * prevTabBT.setActionCommand("PREV_TAB");
         * 
         * buttonPanel.add(prevTabBT);
         * buttonPanel.add(Box.createHorizontalGlue());
         * 
         * nextTabBT = new JButton(">>>>>>");
         * nextTabBT.setBackground(Color.BLUE);
         * nextTabBT.addActionListener(this);
         * nextTabBT.setActionCommand("NEXT_TAB"); buttonPanel.add(nextTabBT);
         */
    }

    void exitConfig() {

        //check if all myNames have certificates
        String badNames = myNamePanel.getNamesWithoutCerts();

        if (badNames != null) {

            int response = JOptionPane.showConfirmDialog(this,
                    "The following names do not have certificates associated with them:\n\n"
                            + badNames
                            + "\nIt is very dangerous to use these names.\n"
                            + "Do you want to specify the certificates now?",
                    "Certificate Missing", JOptionPane.YES_NO_OPTION);
            if (response == JOptionPane.YES_OPTION) {
                return;
            }
        }
        if (changeListener.hasChanged()) {
            int response = JOptionPane.showConfirmDialog(this,
                    "Save any changes to before exiting?");
            if (response == JOptionPane.YES_OPTION) {
                saveConfigFile();
                if (startProxyFlag) {
                    startProxy();
                }
                System.exit(0);
            } else if (response == JOptionPane.NO_OPTION) {
                //NO_OPTION
                System.exit(0);
            }
        } else {

            System.exit(0);
        }

    }

    public void saveConfigFile() {

        outboundPanel.update();
        advancedOptionsPanel.update();

        ListIterator itr = proxyConfigData.overlayList.listIterator();
        while (itr.hasNext()) {

            OCD tmpOverlay = (OCD) itr.next();

            // Update any details still stuck in the wizard
            tmpOverlay.updateFromWizard();
            try {
                tmpOverlay.writeToDisk();

            } catch (IOException ioe) {
                JOptionPane.showMessageDialog(null,
                        "Error while saving configuration details of overlay "
                                + tmpOverlay + ".", "File Save Error",
                        JOptionPane.ERROR_MESSAGE);

            }

        }

        if (proxyConfigFile != null) {
            try {
                proxyConfigData.writeToFile(proxyConfigFile);
            } catch (IOException ioe) {
                JOptionPane.showMessageDialog(this,
                        "Unable to save configuration details to file "
                                + proxyConfigFile + ".", "File Save Error",
                        JOptionPane.ERROR_MESSAGE);
                System.out.println(ioe);
            }
        } else {

            saveAsConfigFile();
        }
    }

    void saveAsConfigFile() {

        int retVal = fc.showSaveDialog(this);

        if (retVal == JFileChooser.APPROVE_OPTION) {
            proxyConfigFile = fc.getSelectedFile();
            saveConfigFile();
            setTitle(windowTitle + " [" + proxyConfigFile + "]");
        }

    }

    public void actionPerformed(ActionEvent ae) {
        String cmd = ae.getActionCommand();

        if (cmd.equals("NEXT_TAB")) {
            int currSelTab = tabPane.getSelectedIndex();
            int maxTabIndex = tabPane.getTabCount() - 1;
            if (currSelTab == maxTabIndex) {
                beep();
            } else {
                currSelTab++;
                tabPane.setSelectedIndex(currSelTab);
            }

        } else if (cmd.equals("PREV_TAB")) {

            int currSelTab = tabPane.getSelectedIndex();
            if (currSelTab == 0) {
                beep();
            } else {
                currSelTab--;
                tabPane.setSelectedIndex(currSelTab);
            }
        }
    }

    public static void beep() {
        Toolkit.getDefaultToolkit().beep();
    }

    public void startProxy() {

        //TODO
    }

    /**
     * Check if this is the first time we are running ProxyConfig
     * 
     * @return true if first time, else false
     */
    private boolean checkFirstTime() {

        File tmpFile = new File(FIRST_TIME_CHECK_FILE);
        if (tmpFile.exists()) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Creates a file to indicate that the ProxyConfig has been invoked
     * previously.
     * 
     * @return
     */
    void setFirstTime() {
        try {
            BufferedWriter bw = new BufferedWriter(new FileWriter(
                    FIRST_TIME_CHECK_FILE));
            if (bw == null) {
                System.out
                        .println("ProxyConfig::setFirstTime() Unable to create "
                                + FIRST_TIME_CHECK_FILE + ".\n");
                return;
            }

            bw
                    .write("Delete this file if you want ProxyConfig to show the wizard on start up.\n");
            bw.close();

        } catch (IOException ioe) {
            System.out
                    .println("ProxyConfig::setFirstTime() Error while writing to "
                            + FIRST_TIME_CHECK_FILE + ".\n");
            ioe.printStackTrace();
        }
    }

    /**
     * 
     * @return true if the current operating system is windows, else returns
     *         false
     */
    public static boolean isWindows() {
        String osName = System.getProperty("os.name", "windows").toLowerCase();
        if (osName.indexOf("windows") != -1) {
            return true;
        } else {
            return false;
        }
    }

    public static boolean isApple() {
	String osName = System.getProperty("os.name", "windows").toLowerCase();
        if (osName.indexOf("mac os x") != -1) {
            return true;
        } else {
            return false;
        }
    }


}

