/*
 * Created on Jan 18, 2005
 */
package proxyconfig;

import proxyconfig.ocd.OCD;
import proxyconfig.outboundconnections.*;
import proxyconfig.mynames.*;
import java.util.*;
import java.io.*;

import proxyconfig.outboundconnections.OutboundRecord;

/**
 * @author danjo
 */
public class ProxyConfigData {

    public static final String MY_NAMES_CONFIG_FILE = "myNamesConfigFile";

    public static final String OUTBOUND_CONNECTION_CONFIG_FILE = "outboundConnectionConfigFile";

    int stdDNSRequestPort;

    int legacyDNSRequestPort;

    String fakeAddrRangeStart;

    String fakeAddrRangeMask;

    String myNameConfigFileName;

    String outboundConfigFileName;

    String winTunName;

    public LinkedList overlayList;

    public ArrayList myNameList;

    public ArrayList outboundList;

    ProxyConfig proxyConfig;
    
    ProxyConfigData() {
        overlayList = new LinkedList();
        outboundList = new ArrayList();
        myNameList = new ArrayList();
    }
  

    void resolveOutboundRecords() {
        ListIterator itr = outboundList.listIterator();
        while (itr.hasNext()) {
            OutboundRecord tmpRecord = (OutboundRecord) itr.next();
            ListIterator hopItr = tmpRecord.hopList.listIterator();
            while (hopItr.hasNext()) {
                HopInfo currHop = (HopInfo) hopItr.next();
                currHop.ocd = getOverlayById(currHop.ocdId);
                if (currHop.ocd == null) {
                    System.out.println("Invalid overlay id " + currHop.ocdId
                            + "specified for HopInfo of  pattern "
                            + tmpRecord.pattern + ".");
                }
            }
        }
    }

    public OCD getOverlayById(String id) {
        ListIterator itr = overlayList.listIterator();
        while (itr.hasNext()) {
            OCD tmpOverlay = (OCD) itr.next();
            if (tmpOverlay.id.equals(id)) {
                return tmpOverlay;
            }
        }
        return null;
    }

    public List getOCDList() {
        return overlayList;
    }

    void writeToFile(File outFile) throws IOException {

        BufferedWriter bw = new BufferedWriter(new FileWriter(outFile));
        bw.write("<?xml version=\"1.0\"?>\n\n");
        bw.write("<ProxyConfigFile \n");

        bw.write("\tstdDNSRequestPort=\"" + stdDNSRequestPort + "\"\n");
        bw.write("\tlegacyDNSRequestPort=\"" + legacyDNSRequestPort + "\"\n");
        bw.write("\tfakeAddrRangeStart=\"" + fakeAddrRangeStart + "\"\n");
        bw.write("\tfakeAddrRangeMask=\"" + fakeAddrRangeMask + "\"\n");

        bw.write("\t" + MY_NAMES_CONFIG_FILE + "=\"" + myNameConfigFileName
                + "\"\n");
        bw.write("\t" + OUTBOUND_CONNECTION_CONFIG_FILE + "=\""
                + outboundConfigFileName + "\"\n");

        bw.write(" >\n\n\n");

        bw.write("<OC-D-List>\n");

        ListIterator oIter = overlayList.listIterator();
        while (oIter.hasNext()) {
            OCD tmpOverlay = (OCD) oIter.next();
            bw.write("\t<OC-D\n");
            bw.write("\t\ttype=\"" + entitizeXML(tmpOverlay.type) + "\"\n");
            bw.write("\t\tid=\"" + entitizeXML(tmpOverlay.id) + "\"\n");
            bw.write("\t\tdescription=\"" + entitizeXML(tmpOverlay.description)
                    + "\"\n");
            bw.write("\t\tconfigFile=\"" + entitizeXML(tmpOverlay.configFile)
                    + "\"\n");
            bw.write("\t\tlibFileName_windows=\""
                    + entitizeXML(tmpOverlay.libFileName_windows) + "\"\n");
            bw.write("\t\tlibFileName_linux=\""
                    + entitizeXML(tmpOverlay.libFileName_linux) + "\"\n");
            bw.write("\t\tlibFileName_apple=\""
                    + entitizeXML(tmpOverlay.libFileName_apple) + "\"\n");

            if (tmpOverlay.isActive) {
                bw.write("\t\tactive=\"yes\"\n");
            } else {
                bw.write("\t\tactive=\"no\"\n");
            }
            Iterator propItr = tmpOverlay.propertyMap.keySet().iterator();
            while (propItr.hasNext()) {
                String currKey = (String) propItr.next();
                String currProp = (String) tmpOverlay.propertyMap.get(currKey);
                bw.write("\t\t" + currKey + "=\"" + currProp + "\"\n");
            }
            bw.write("\t/>\n\n");
        }
        bw.write("</OC-D-List>\n\n\n");

        bw.write("<Tunnel>\n");
        if (winTunName != null) {
            bw.write("\t<WinTAPName>" + winTunName + "</WinTAPName>\n");
        }

	String tunDevicePath = null;
	if (ProxyConfig.isApple()) {
	   tunDevicePath = "/dev/tun0";
        } else {
	   tunDevicePath = "/dev/net/tun";
        }	
	bw.write (
        		"\t<DevicePath>" + tunDevicePath + "</DevicePath>\n" +
        		"\t<DeviceName>tun0</DeviceName>\n" +
        		"\t<MTU>1200</MTU>\n" +
        		"\t<IPAddress>10.0.0.1</IPAddress>\n" +
        		"\t<RoutingTable>tuntab</RoutingTable>\n" +
        		"\t<ifconfigCmd>/sbin/ifconfig</ifconfigCmd>\n" +
        		"\t<iptablesCmd>/sbin/iptables</iptablesCmd>\n" +
        		"\t<ipCmd>/sbin/ip</ipCmd>\n" +
        		"\t<modprobeCmd>/sbin/modprobe</modprobeCmd>\n" +
			"\t<AppleCapturePort>13346</AppleCapturePort>\n" + 
    			"</Tunnel>\n\n"
		);
    
	bw.write (
			"<Security\n" +
            		"\tSecurityLevel=\"1\"\n" +
			"\tCA_CertificateFile=\"config_files/certificates/CA_cert.pem\"\n" +
        		"/>\n\n"
		);
/*
        BufferedReader br = new BufferedReader(new FileReader(
                "config_files/tun-config.xml"));
        String tmpLine;
        while ((tmpLine = br.readLine()) != null) {
            bw.write(tmpLine + "\n");
        }
        br.close();
*/
        bw.write("</ProxyConfigFile>\n");

        bw.close();

        writeMyNameList();
        writeOutboundList();
    }

    private void writeMyNameList() throws IOException {
        BufferedWriter bw = new BufferedWriter(new FileWriter(
                myNameConfigFileName));
        bw.write("<?xml version=\"1.0\"?>\n\n");
        bw.write("<MyNameList>\n");

        ListIterator nameIter = myNameList.listIterator();
        while (nameIter.hasNext()) {
            MyName currName = (MyName) nameIter.next();

            bw.write("\t<MyName\n");

            bw.write("\t\tname=\"" + entitizeXML(currName.name) + "\"\n");
            bw.write("\t\tocdId=\"" + entitizeXML(currName.ocdId) + "\"\n");
            bw.write("\t\tactive=\"" + currName.active + "\"\n");

            if (currName.certOKFlag == MyName.CERT_VALID) {

                bw.write("\t\tcertificate=\"" + currName.certFileName + ":"
                        + currName.privKeyFileName + "\"\n");
            }

            Iterator propItr = currName.ocdSpecificProperties.keySet()
                    .iterator();
            while (propItr.hasNext()) {
                String currKey = (String) propItr.next();
                String currProp = (String) currName.ocdSpecificProperties
                        .get(currKey);
                bw.write("\t\t" + currKey + "=\"" + currProp + "\"\n");
            }
            bw.write("\t/>\n\n");
        }
        bw.write("</MyNameList>\n");

        bw.close();

    }

    private void writeOutboundList() throws IOException {
        BufferedWriter bw = new BufferedWriter(new FileWriter(
                outboundConfigFileName));
        bw.write("<?xml version=\"1.0\"?>\n\n");
        bw.write("<OutboundPathConfig>\n");

        ArrayList tmpOutList = new ArrayList(outboundList);
        // for each overlay, add its suffix pattern->overlay mapping
        ListIterator ocdItr = overlayList.listIterator();
        while (ocdItr.hasNext()) {
            OCD currOCD = (OCD) ocdItr.next();
            OutboundRecord tempRec = new OutboundRecord();
            tempRec.compressionAlgo = OutboundRecord.COMPRESSION_ALGO_NONE;
            tempRec.compressionLevel = -1;
            tempRec.securityProtocol = OutboundRecord.SECURITY_PROTOCOL_NONE;
            tempRec.pattern = "*." + currOCD.id;
            tempRec.ocdDefaultFlag = true;

            HopInfo tmpHop = new HopInfo();
            tmpHop.ocd = currOCD;
            tmpHop.ocdId = currOCD.id;
            tmpHop.endPointName = null;
            tempRec.hopList.add(tmpHop);

            tmpOutList.add(0, tempRec);
        }

        ListIterator pathIter = tmpOutList.listIterator();
        while (pathIter.hasNext()) {
            OutboundRecord currRec = (OutboundRecord) pathIter.next();

            bw.write("\t<PathInfo\n");
            if (currRec.ocdDefaultFlag) {
                bw.write("\t\tocdDefaultFlag=\"true\"\n");
            } else {
                bw.write("\t\tocdDefaultFlag=\"false\"\n");
            }

            bw.write("\t>\n");

            bw.write("\t\t<Match urlPattern=\"" + entitizeXML(currRec.pattern)
                    + "\"/>\n");
            bw.write("\t\t<Security protocol=\""
                    + entitizeXML(currRec.securityProtocol) + "\"/>\n");
            bw.write("\t\t<Compression algo=\""
                    + entitizeXML(currRec.compressionAlgo) + "\" level=\""
                    + currRec.compressionLevel + "\"/>\n");
            ;

            Iterator hopItr = currRec.hopList.listIterator();
            while (hopItr.hasNext()) {
                HopInfo currHop = (HopInfo) hopItr.next();
                bw.write("\t\t<Hop overlayId=\"" + currHop.ocdId + "\"");
                if (currHop.endPointName != null
                        && !currHop.endPointName.trim().equals("")) {
                    bw.write(" hopEndPointName=\"" + currHop.endPointName
                            + "\"");
                }
		if (!currHop.authenticateFlag) {
		    bw.write (" authenticate=\"no\" ");
		}
                bw.write(" />\n");

            }
            bw.write("\t</PathInfo>\n\n");
        }
        bw.write("</OutboundPathConfig>\n");

        bw.close();

    }

    /** Converts stuff like ", <, > etc to &quot;, &lt, &gt etc. */
    public static String entitizeXML(String str) {

        if (str == null) {
            return "";
        }

        str = str.replaceAll("&", "&amp;"); // this should be the first
                                            // replacement
        str = str.replaceAll("\"", "&quot;");
        str = str.replaceAll("<", "&lt;");
        str = str.replaceAll(">", "&gt;");

        return str;
    }

    public String availableOCDSuffixes() {

        String retVal = "";

        ListIterator itr = overlayList.listIterator();
        while (itr.hasNext()) {
            OCD o = (OCD) itr.next();
            retVal += o.id;
            if (itr.hasNext()) {
                retVal += ", ";
            }
        }
        return retVal;
    }

    public void addOCD(OCD o) {
        overlayList.add(o);
    }

    // TODO Check if the name being added already exists.
    public void addMyName(MyName n) {
        myNameList.add(n);
    }

    public void delOutbound (OutboundRecord rec) {
        outboundList.remove(rec);
        proxyConfig.outboundPanel.delRecord (rec);
    }
    /**
     * 
     * @param rec
     * @return false if pattern already exists. TODO Better regex matching.
     */
    public boolean addOutbound(OutboundRecord rec) {
        Iterator itr = outboundList.iterator();

        String newPattern = rec.pattern.toLowerCase();

        while (itr.hasNext()) {
            OutboundRecord currRec = (OutboundRecord) itr.next();
            String currPattern = currRec.pattern.toLowerCase();
            if (currPattern.startsWith("*")) {
                String tmpSuffix = currPattern.substring(1);
                if (newPattern.endsWith(tmpSuffix)) {
                    return false;
                }
            } else {
                if (currPattern.equals(newPattern)) {
                    return false;
                }
            }
            // check dst port
            if (currRec.dstPort > 0 && currRec.dstPort == rec.dstPort) {
                return false;
            }

        }

        this.outboundList.add(rec);
        proxyConfig.outboundPanel.addRecord(rec);
        
        return true;
    }
}
