/*
 * Created on Jan 22, 2005
 */
package proxyconfig.i3;

import javax.swing.*;
import javax.swing.table.TableColumn;

import proxyconfig.InputChangeListener;
import proxyconfig.ProxyConfig;
import proxyconfig.TableSorter;

import java.awt.*;
import java.awt.event.*;

/**
 * @author danjo
 */
public class I3AddressBookPanel extends JPanel implements ActionListener {

    I3OCD overlay;
    I3AddressBookTableModel i3AddressBookTableModel;
    
    JTable i3AddressBookTable;
    InputChangeListener changeListener;
    
    I3AddressBookPanel (I3OCD overlay, InputChangeListener changeListener) {
        this.overlay = overlay;
        this.changeListener = changeListener;
        
        
        setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
    
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        
        String[] columns = {"URL/I3 Name", "I3 Id"};
        
        i3AddressBookTableModel = new I3AddressBookTableModel(columns);
        
        TableSorter tableSorter = new TableSorter(i3AddressBookTableModel);
        i3AddressBookTable = new JTable(tableSorter);
		tableSorter.addMouseListenerToHeaderInTable(i3AddressBookTable);

        i3AddressBookTable.setBackground(proxyconfig.ProxyConfig.WHITE_COLOR);
        i3AddressBookTableModel.addRecords(overlay.addressBook.addressList);
        i3AddressBookTable.setPreferredSize(new Dimension(700, 400));
        
        i3AddressBookTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
        TableColumn column = null;
        column = i3AddressBookTable.getColumnModel().getColumn(0);
        column.setPreferredWidth(150);

        column = i3AddressBookTable.getColumnModel().getColumn(1);
        column.setPreferredWidth(500);

        JScrollPane scrollPane = new JScrollPane(
                	i3AddressBookTable,
                	JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                	JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED
                	);
        scrollPane.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        scrollPane.setPreferredSize(new Dimension(700, 400));

        add(scrollPane);
        
        add(Box.createRigidArea(new Dimension(10,10)));
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        add(buttonPanel);
        
        buttonPanel.add(Box.createHorizontalGlue());
        
        JButton addBT = new JButton("Add new mapping");
        addBT.setActionCommand("ADD_MAPPING");
        addBT.addActionListener(this);
        buttonPanel.add(addBT);

        buttonPanel.add(Box.createRigidArea(new Dimension(5,5)));

        JButton editBT = new JButton("Edit selected mapping");
        editBT.setActionCommand("EDIT_MAPPING");
        editBT.addActionListener(this);
        buttonPanel.add(editBT);

        buttonPanel.add(Box.createRigidArea(new Dimension(5,5)));

        JButton delBT = new JButton("Delete selected mapping");
        delBT.setActionCommand("DEL_MAPPING");
        delBT.addActionListener(this);
        buttonPanel.add(delBT);

        buttonPanel.add(Box.createHorizontalGlue());

        add(Box.createRigidArea(new Dimension(10,10)));
    }
    
    public void actionPerformed (ActionEvent ae) {
        String cmd = ae.getActionCommand();
        
        if (cmd.equals("ADD_MAPPING")) {

            ProxyConfig.changeOptionPaneBackground(Color.WHITE);
            I3AddressBookEntry currEntry = new I3AddressBookEntry();
                
            I3AddressBookEntryPanel i3AddressBookEntryPanel = new I3AddressBookEntryPanel();
            i3AddressBookEntryPanel.setEntry(currEntry);
                
            int retVal = JOptionPane.showConfirmDialog(null, 
                    i3AddressBookEntryPanel, "Add new Address Book entry", JOptionPane.OK_CANCEL_OPTION);
            if (retVal == JOptionPane.OK_OPTION) {
                
                if (i3AddressBookEntryPanel.checkFields()) {
                    i3AddressBookEntryPanel.updateEntry(currEntry);
                
                	if (overlay.addressBook.contains (currEntry)) {
                 
                    	ProxyConfig.restoreOptionPaneBackground();
                    	JOptionPane.showMessageDialog(null, 
                            	"An entry for the name \"" + currEntry.name + "\" already exists in the address book.\n" +
                            	"There cannot be two entries for the same name.  Please modify the existing entry.",
                            	"Duplicate name",
                            	JOptionPane.ERROR_MESSAGE
                    		);
                    	return;
                	} else {
                    	i3AddressBookTableModel.addRecord(currEntry);
                    	overlay.addressBook.addressList.add(currEntry);
                    	changeListener.setChanged(true);
                	}
                
                }
            }
                
            ProxyConfig.restoreOptionPaneBackground();
        } else if (cmd.equals("EDIT_MAPPING")) {

            ProxyConfig.changeOptionPaneBackground(Color.WHITE);
            I3AddressBookEntry currEntry = (I3AddressBookEntry) i3AddressBookTableModel.getRecord(i3AddressBookTable.getSelectedRow());
            if (currEntry != null) {
                I3AddressBookEntryPanel i3AddressBookEntryPanel = new I3AddressBookEntryPanel();
                i3AddressBookEntryPanel.setEntry(currEntry);
                
                
                int retVal = JOptionPane.showConfirmDialog(null, 
                        i3AddressBookEntryPanel, "Modify Address Book entry", JOptionPane.OK_CANCEL_OPTION);
                if (retVal == JOptionPane.OK_OPTION) {
                    if (i3AddressBookEntryPanel.checkFields()) {
                        i3AddressBookEntryPanel.updateEntry(currEntry);
                        i3AddressBookTableModel.fireTableDataChanged();
                        changeListener.setChanged(true);
                    }
                }
                
            } else {
                ProxyConfig.beep();
            }
            ProxyConfig.restoreOptionPaneBackground();


        } else if (cmd.equals("DEL_MAPPING")) {

            System.out.println("--> " + i3AddressBookTable.getSelectedRow());
            
            I3AddressBookEntry currEntry = (I3AddressBookEntry) i3AddressBookTableModel.getRecord(i3AddressBookTable.getSelectedRow());

            if (currEntry != null) {
                ProxyConfig.restoreOptionPaneBackground();
                int retVal = JOptionPane.showConfirmDialog(null, 
                        "Are you sure you want to delete the mapping\n" + currEntry + " ?", 
                        "Confirm Delete", JOptionPane.YES_NO_OPTION);
                if (retVal == JOptionPane.YES_OPTION) {
                    i3AddressBookTableModel.deleteRecord(currEntry);
                    overlay.addressBook.addressList.remove(currEntry);
                    changeListener.setChanged(true);
                }
                
            } else {
                ProxyConfig.beep();
            }
        
        }
    }
    
    void update() {
        overlay.addressBook.addressList = i3AddressBookTableModel.getRecordList();
    }
}
