/*
 * Created on May 16, 2005
 */
package proxyconfig.mynames;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import javax.swing.*;
import javax.swing.border.BevelBorder;

import java.util.*;

import proxyconfig.certificate.*;
import proxyconfig.*;
import proxyconfig.ocd.OCD;

/**
 * @author danjo
 */
public class MyNameEditPanel extends JPanel implements  ItemListener {

    JTextField nameTF;
    JComboBox ocdCombo;
    
    
    CertificateEditPanel certificateEditPanel;
    
    ProxyConfig proxyConfig;
    
    /** Map of name edit panels keyed by OCD type */
    HashMap nameEditPanel_type_map;
    
    public MyNameEditPanel(MyName currName, ProxyConfig proxyConfig) {
    
        this.proxyConfig = proxyConfig;
        
        setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        setLayout (new BoxLayout (this, BoxLayout.Y_AXIS));
        
        JPanel mainPanel = new JPanel();
        mainPanel.setBackground (proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        add (mainPanel);
        

        // Initialize the name edit panels for the different OCD types.
        nameEditPanel_type_map = new HashMap();
        ArrayList tmpOCDTypeList = proxyConfig.guiConfigReader.getOCDTypeList();
        Iterator itr = tmpOCDTypeList.iterator();
        while (itr.hasNext()) {
            String tmpType = (String) itr.next();
            OCD tmpOCD = proxyConfig.guiConfigReader.getNewOCD(tmpType);
            OCDNameEditPanel tmpNameEditPanel = tmpOCD.getNewNameEditPanel (currName);
            if (tmpNameEditPanel != null) {
                nameEditPanel_type_map.put (tmpType, tmpNameEditPanel);
            }
        }
        
        
    	//Construct the GUI
		Insets insets = new Insets(2,2,2,2);
		GridBagLayout gbag = new GridBagLayout();
		mainPanel.setLayout(gbag);

		GridBagConstraints gc = new GridBagConstraints();
		gc.weighty = 0.0;
		gc.fill = GridBagConstraints.HORIZONTAL;
		gc.insets = insets;
		gc.gridx = GridBagConstraints.REMAINDER;
		gc.anchor = GridBagConstraints.WEST;

		JLabel tempLabel;

		// The preferred Dimension of all the input fields - mainly used to enforce a common height
		Dimension minDim = new Dimension(50, 25);
		
		tempLabel = new JLabel("Name/URL");
		tempLabel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		gc.gridx = 0;
		gc.gridy = 0;
		gc.weightx = 0.0;
		gbag.setConstraints(tempLabel, gc);
		mainPanel.add(tempLabel);

		nameTF = new JTextField(40);
		nameTF.setMinimumSize(minDim);
		nameTF.setBackground(ProxyConfig.INPUT_BOX_COLOR);
		gc.gridx = 1;
		gc.gridy = 0;
		gc.weightx = 1.0;
		gbag.setConstraints(nameTF, gc);
		mainPanel.add(nameTF);

		tempLabel = new JLabel("Overlay");
		tempLabel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		gc.gridx = 0;
		gc.gridy = 1;
		gc.weightx = 0.0;
		gbag.setConstraints(tempLabel, gc);
		mainPanel.add(tempLabel);
	    
		ocdCombo = new JComboBox();
		ocdCombo.setBackground(ProxyConfig.INPUT_BOX_COLOR);
		ocdCombo.setMinimumSize(minDim);
		ocdCombo.addItemListener(this);
		
		gc.gridx = 1;
		gc.gridy = 1;
		gc.weightx = 1.0;
		gbag.setConstraints(ocdCombo, gc);
		mainPanel.add(ocdCombo);

		Iterator oitr = proxyConfig.proxyConfigData.overlayList.listIterator();
		while (oitr.hasNext()) {
		    OCD currOCD = (OCD) oitr.next();
		    ocdCombo.addItem(currOCD);
		}

		JPanel tmpPanel;

		certificateEditPanel = new CertificateEditPanel(proxyConfig, currName, nameTF);
		add (certificateEditPanel);
		
	    		
		// Set all the name edit panels to be invisible.
		itr = nameEditPanel_type_map.values().iterator();
		while (itr.hasNext()) {
		    
		    OCDNameEditPanel tmpNameEditPanel = (OCDNameEditPanel) itr.next();
		    tmpNameEditPanel.setVisible(false);
		    add (tmpNameEditPanel);
		}
		
		// Set the first name edit panel to visible
		itr = nameEditPanel_type_map.values().iterator();
		if (itr.hasNext()) {
		    OCDNameEditPanel tmpNameEditPanel = (OCDNameEditPanel) itr.next();
		    tmpNameEditPanel.setVisible(true);		    
		}
		
		add(Box.createVerticalGlue());
		
		if (currName != null) {
		    
		    nameTF.setText(currName.name);
		    ocdCombo.setSelectedItem(currName.ocd);
		}
    }
    
    public Dimension getPreferredSize() {
        
        return new Dimension(600,350);
    }

    
    
    boolean checkFields() {
        boolean retVal = true;
        
        String msg = "";

        if (nameTF.getText().trim().equals("")) {
            retVal = false;
            msg += "Name field should not be empty.\n";
        }
        
        OCD selOCD = (OCD) ocdCombo.getSelectedItem ();
        
        OCDNameEditPanel tmpNameEditPanel = getNameEditPanel(selOCD.type);
        if (tmpNameEditPanel != null) {
               
            String retStr = tmpNameEditPanel.checkFields();
            if (!retStr.equals("")) {
                
                msg += "\n" + retStr;
                retVal = false;
            } 
        }
        
        if (!retVal) {
            JOptionPane.showMessageDialog( null,
                    msg,
                    "Error in Name",
                    JOptionPane.ERROR_MESSAGE
        		);
        }
        return retVal;
    }

    /* (non-Javadoc)
     * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
     */
    public void itemStateChanged(ItemEvent e) {
        
        OCD selOCD = (OCD) ocdCombo.getSelectedItem();
        
        // Set all the name edit panels to invisible
        Iterator itr = nameEditPanel_type_map.values().iterator();
        while (itr.hasNext()) {
            OCDNameEditPanel tmpNameEditPanel = (OCDNameEditPanel) itr.next();
            tmpNameEditPanel.setVisible (false);
        }
        
        // Set the name edit panel corresponding to the selected ocd type to visible.
        OCDNameEditPanel tmpNameEditPanel = getNameEditPanel(selOCD.type);
        if (tmpNameEditPanel != null) {
            tmpNameEditPanel.setVisible (true);
        }
    }
    
    public void updateRecord (MyName rec) {
        
        rec.name = nameTF.getText();
        OCD tmpOCD = (OCD) ocdCombo.getSelectedItem();
        rec.ocd = tmpOCD;
        rec.ocdId = tmpOCD.id;
        
        rec.privKeyFileName = certificateEditPanel.getPrivKeyFileName();
        rec.certFileName = certificateEditPanel.getCertificateFileName();

        OCDNameEditPanel tmpNameEditPanel = getNameEditPanel (tmpOCD.type);
        if (tmpNameEditPanel != null) {
            tmpNameEditPanel.update (rec);
        }
    }
    

    private OCDNameEditPanel getNameEditPanel (String type) {
        
        return (OCDNameEditPanel) nameEditPanel_type_map.get(type);
    }


}