/*
 * Created on May 18, 2005
 */
package proxyconfig.ocd;

import javax.swing.table.AbstractTableModel;

import proxyconfig.outboundconnections.OutboundRecord;
import proxyconfig.outboundconnections.OutboundTable;

import java.awt.Toolkit;
import java.util.*;
/**
 * @author danjo
 * This class implements the TableModel to display Records 
 */
public class OCDTableModel extends AbstractTableModel {

	/** The list of records to displayed in the table */
	private ArrayList recordList;
	
	/** The label for the columns */
	private String[] columns;
	
	/** A reference to the my name Table of which this object is the data model. */
	OCDTable recordTable;

	public OCDTableModel(String[] columns) {
		
		recordList = new ArrayList();
		this.columns = columns;
		 	
	}
	
	public int getRowCount() {
		
		return recordList.size();
	}

	public int getColumnCount() {
		
		return columns.length;		
	}
	
	public String getColumnName(int columnIndex) {
		return columns[columnIndex];
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		
		OCD currRec = (OCD) recordList.get(rowIndex);
		
		switch (columnIndex) {
			case 0:
			    return currRec.id;
			
			case 1:
			    return currRec.type;
		
			case 2:
			    return currRec.description;
		
			default:
			    return "error";
		}
	}
	
	/** Add a record to the table */
	public void addRecord(OCD rec) {
		
		recordList.add(rec);
		fireTableDataChanged();
	}
	
	/** Add all the records in the specified list to the table */
	public void addRecords(LinkedList rList) {
		
		ListIterator itr = rList.listIterator();
		
		while (itr.hasNext()) {
			OCD tempRec = (OCD) itr.next();
			recordList.add(tempRec);
		}
		fireTableDataChanged();

	}

	/** Remove all records */
	public void clear() {
		recordList.clear();
		fireTableDataChanged();
	}
	
	/** Returns the specified record 
	 * Class cast errors possible here. 
	 * @param recIndex the Index of the Record in the records list
	 * @param row the selected row (may be same as recIndex, not sure)
	 */
	public OCD getRecord(int recIndex) {
	
		if (recIndex >= 0 && recIndex < recordList.size()) {
			OCD tempRec = (OCD) recordList.get(recIndex);
			return tempRec;
		}
		
		return null;
	}


	/** Returns the specified record 
	 * Class cast errors possible here.
	 * Journal Issues not specially handled. 
	 * @param recIndex the Index of the Record in the records list
	 * @param row the selected row (may be same as recIndex, not sure)
	 */
	public OCD getRecordOriginal(int recIndex) {
	
		if (recIndex >= 0 && recIndex < recordList.size()) {
			OCD tempRec = (OCD) recordList.get(recIndex);
			return (OCD) recordList.get(recIndex);
		}
		
		return null;
		
	}

	
	/** Returns a list of all records contained in this table */
	public ArrayList getRecordList() {
		return new ArrayList(recordList);
	}

	/** Returns the inded of the specified Record in the records list */
	public int indexOf(OutboundRecord rec) {
		return recordList.indexOf(rec);
	}

	public void moveRecordUp(int recNum) {
	    
        if (recNum > 0) {
    	    OCD tmpRecord = (OCD) recordList.get(recNum);
    	    recordList.remove(recNum);
    	    int moveTo = recNum - 1;
    	    recordList.add(moveTo, tmpRecord);
    	    fireTableDataChanged();
    	    
    	    recordTable.setRowSelectionInterval(moveTo, moveTo);

        } else {
            Toolkit.getDefaultToolkit().beep();
        }

	    
	}

	public void moveRecordDown(int recNum) {
	    
        if (recNum >= 0 && recNum < getRowCount() - 1) {
    	    OCD tmpRecord = (OCD) recordList.get(recNum);
    	    recordList.remove(recNum);
    	    int moveTo = recNum + 1;
    	    recordList.add(moveTo, tmpRecord);

    	    fireTableDataChanged();
    	    
    	    recordTable.setRowSelectionInterval(moveTo, moveTo);

        } else {
            Toolkit.getDefaultToolkit().beep();
        }

	}

	public void removeRecord(OCD rec) {
	    
	    recordList.remove(rec);
	    fireTableDataChanged();
	}
}
