/*
 * Created on Jan 22, 2005
 */
package proxyconfig.outboundconnections;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.*;

import proxyconfig.ProxyConfig;
import proxyconfig.ProxyConfigData;
import proxyconfig.ocd.OCD;

import java.awt.*;

import java.util.*;

/**
 * @author danjo
 */
public class OutboundEditPanel extends JPanel {

    JTextField patternTF;
    JComboBox securityAlgoCombo;
    JComboBox compressionAlgoCombo;
    JTextField compressionLevelTF;
    
    HopPanel hopPanel;
    ProxyConfig proxyConfig;
    
    OutboundEditPanel(ProxyConfig proxyConfig) {
        
        this.proxyConfig = proxyConfig;
        
        ProxyConfigData proxyConfigData = proxyConfig.proxyConfigData;
        
        setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
        
    	//Construct the GUI
		Insets insets = new Insets(2,2,2,2);
		GridBagLayout gbag = new GridBagLayout();
		setLayout(gbag);

		GridBagConstraints gc = new GridBagConstraints();
		gc.weighty = 0.0;
		gc.fill = GridBagConstraints.HORIZONTAL;
		gc.insets = insets;
		gc.gridx = GridBagConstraints.REMAINDER;
		gc.anchor = GridBagConstraints.WEST;

		
		JLabel tempLabel;

		// The preferred Dimension of all the input fields - mainly used to enforce a common height
		Dimension minDim = new Dimension(50, 25);
		
		tempLabel = new JLabel("URL Pattern");
		tempLabel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		gc.gridx = 0;
		gc.gridy = 0;
		gc.weightx = 0.0;
		gbag.setConstraints(tempLabel, gc);
		add(tempLabel);

		patternTF = new JTextField(20);
		patternTF.setMinimumSize(minDim);
		patternTF.setBackground(ProxyConfig.INPUT_BOX_COLOR);
		
		gc.gridx = 1;
		gc.gridy = 0;
		gc.weightx = 1.0;
		gbag.setConstraints(patternTF, gc);
		add(patternTF);

		tempLabel = new JLabel("Security Algorithm");
		tempLabel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		gc.gridx = 0;
		gc.gridy = 1;
		gc.weightx = 0.0;
		gbag.setConstraints(tempLabel, gc);
		add(tempLabel);

		JPanel securityPanel = new JPanel ();
		securityPanel.setBackground (proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		securityPanel.setLayout (new BoxLayout (securityPanel, BoxLayout.X_AXIS));
		
		securityAlgoCombo = new JComboBox();
		securityAlgoCombo.setMinimumSize(minDim);
		securityAlgoCombo.setBackground(ProxyConfig.INPUT_BOX_COLOR);
		securityPanel.add (securityAlgoCombo);
		securityPanel.add (Box.createRigidArea (new Dimension (10,10)));
		
		securityAlgoCombo.addItem (OutboundRecord.SECURITY_PROTOCOL_NONE);
		securityAlgoCombo.addItem (OutboundRecord.SECURITY_PROTOCOL_CUSTOM_SSL);
		
		gc.gridx = 1;
		gc.gridy = 1;
		gc.weightx = 1.0;
		gbag.setConstraints(securityPanel, gc);
		add(securityPanel);


		tempLabel = new JLabel("Compression Algorithm");
		tempLabel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		gc.gridx = 0;
		gc.gridy = 2;
		gc.weightx = 0.0;
		gbag.setConstraints(tempLabel, gc);
		add(tempLabel);

		JPanel compressionPanel = new JPanel ();
		compressionPanel.setBackground (proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		compressionPanel.setLayout (new BoxLayout (compressionPanel, BoxLayout.X_AXIS));
		
		compressionAlgoCombo = new JComboBox();
		compressionAlgoCombo.setMinimumSize(minDim);
		compressionAlgoCombo.setBackground(ProxyConfig.INPUT_BOX_COLOR);
		compressionPanel.add (compressionAlgoCombo);
		compressionPanel.add (Box.createRigidArea (new Dimension (10,10)));
		
		compressionAlgoCombo.addItem (OutboundRecord.COMPRESSION_ALGO_NONE);
		compressionAlgoCombo.addItem (OutboundRecord.COMPRESSION_ALGO_ZLIB);
		
		tempLabel = new JLabel ("Compression Level");
		tempLabel.setBackground (proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		compressionPanel.add (tempLabel);
		compressionPanel.add (Box.createRigidArea (new Dimension(5,5)));
		
		compressionLevelTF = new JTextField (5);
		compressionLevelTF.setMinimumSize (minDim);
		compressionLevelTF.setBackground (ProxyConfig.INPUT_BOX_COLOR);
		compressionPanel.add (compressionLevelTF);
		
		gc.gridx = 1;
		gc.gridy = 2;
		gc.weightx = 1.0;
		gbag.setConstraints(compressionPanel, gc);
		add(compressionPanel);

		
		tempLabel = new JLabel("Path");
		tempLabel.setBackground(proxyconfig.ProxyConfig.BACKGROUND_COLOR);
		gc.gridx = 0;
		gc.gridy = 3;
		gc.weightx = 0.0;
		gbag.setConstraints(tempLabel, gc);
		add(tempLabel);
	    
		hopPanel = new HopPanel(proxyConfig);
		
		gc.gridx = 1;
		gc.gridy = 3;
		gc.weightx = 1.0;
		gbag.setConstraints(hopPanel, gc);
		add(hopPanel);
    }
    
    void setRecord(OutboundRecord rec) {
        patternTF.setText(rec.pattern);
        securityAlgoCombo.setSelectedItem(rec.securityProtocol);
        compressionAlgoCombo.setSelectedItem (rec.compressionAlgo);
        compressionLevelTF.setText(rec.compressionLevel + "");
        
        ListIterator itr = rec.hopList.listIterator();
        while (itr.hasNext()) {
            HopInfo currHop = (HopInfo) itr.next();
            hopPanel.addHop (currHop);
        }
    }
        
    
    boolean updateRecord(OutboundRecord rec) {
        
        rec.pattern = patternTF.getText();
        
        
        rec.securityProtocol = (String) securityAlgoCombo.getSelectedItem();
        rec.compressionAlgo = (String) compressionAlgoCombo.getSelectedItem();
        
        try {
            
            String tmpCompressionLevel = compressionLevelTF.getText();
            if (tmpCompressionLevel == null || tmpCompressionLevel.trim().equals("")) {
                rec.compressionLevel = 0;
            } else {
                rec.compressionLevel = Integer.parseInt (compressionLevelTF.getText());
            }
        } catch (NumberFormatException nfe) {
            JOptionPane.showMessageDialog(
            		null, 
            		"Invalid compression level " + compressionLevelTF.getText(), 
            		"Invalid Entry", 
            		JOptionPane.WARNING_MESSAGE
            	);
           
            return false;
        }
        
        rec.hopList.clear();
        for (int hopIndex = 0; hopIndex < hopPanel.listModel.size(); hopIndex++) {
            HopInfo currHop = (HopInfo)hopPanel.listModel.getElementAt (hopIndex);
            rec.hopList.add(currHop);
        }
        return true;
    
    }

    boolean checkFields() {

        String tmpPattern = patternTF.getText().trim();
        
        boolean retVal = true;
        String msg = "";
        
        if (tmpPattern.equals ("")) {
        
            retVal = false;
            msg += "Pattern field should not be blank,\n";
            
        }
        
        if (hopPanel.listModel.getSize() < 1) {
            retVal = false;
            msg += "At least one hop should be specified.\n";
        } else {
        
            OCD suffixMatchOCD = null;
        
        	// go through all the ocds and check if the pattern matches *.ocdid */
        	ListIterator itr = proxyConfig.proxyConfigData.overlayList.listIterator();
        	while (itr.hasNext()) {
            	OCD currOCD = (OCD) itr.next();
            	if (tmpPattern.endsWith("*." + currOCD.id)) {
                	suffixMatchOCD = currOCD;
            	}
        	}
        
        	HopInfo tmpLastHop = (HopInfo) hopPanel.listModel.lastElement();
        	if (suffixMatchOCD != null && tmpLastHop.ocd != suffixMatchOCD) {
            	retVal = false;
            	msg += "The pattern ends with \'*." + suffixMatchOCD.id + "\'.\n" +
            			"However the last hop does not use the corresponding overlay.";
            	
        	}
        	
        }
        
        if (hopPanel.listModel.size() > 0) {
            HopInfo tmpLastHop = (HopInfo) hopPanel.listModel.lastElement();
        	if (tmpLastHop.endPointName != null && !tmpLastHop.endPointName.trim().equals("")) {
            	retVal = false;
            	msg += "The last hop of the path should not be a middle box.";
        	}
        }
        
        if (!retVal) {
            JOptionPane.showMessageDialog (proxyConfig, msg, "Error", JOptionPane.ERROR_MESSAGE);
        }
        
        return retVal;

    }


}