/*
 * Created on Apr 12, 2006
 */
package proxyconfig.rsptunnelocd;

import java.util.*;
import javax.swing.table.*;

import proxyconfig.outboundconnections.*;

/**
 * @author danjo
 */
public class URLPatternTableModel extends AbstractTableModel {
	/** The list of records to displayed in the table */
	private ArrayList recordList;
	
	/** The label for the columns */
	private String[] columns;
	
	public URLPatternTableModel(String[] columns) {
		
		recordList = new ArrayList();
		this.columns = columns;
		 	
	}
	
	public int getRowCount() {
		
		return recordList.size();
	}

	public int getColumnCount() {
		
		return columns.length;		
	}
	
	public String getColumnName(int columnIndex) {
		return columns[columnIndex];
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		
		OutboundRecord currRec = (OutboundRecord) recordList.get(rowIndex);
		switch (columnIndex) {
			case 0:
			    return currRec.pattern;
			
			case 1:
			{
			    if (currRec.hopList.size() > 0) {
			        HopInfo firstHop = (HopInfo) currRec.hopList.get(0);
			    	return firstHop.endPointName;	
			    } else {
			        return "ERROR IN HOP";
			    }
			}
			
			default:
			    return "UNKNOWN COLUMN";
			
		}
	}
	
	/** Add a record to the table */
	public void addRecord(Object rec) {
	    
	    recordList.add(rec);
	    fireTableDataChanged();
	}
	
	
	public void deleteRecord(Object rec) {
	    recordList.remove(rec);
	    fireTableDataChanged();
	}
	
	/** Add all the records in the specified list to the table */
	public void addRecords(List rList) {
		
		ListIterator itr = rList.listIterator();
		
		while (itr.hasNext()) {
		    String tempRec = (String) itr.next();
			recordList.add(tempRec);
		}
	    fireTableDataChanged();
	}
	/** Remove all records */
	public void clear() {
		recordList.clear();
		fireTableDataChanged();
	}
	
	/** Returns the specified record 	 * Class cast errors possible here. 
	 * @param recIndex the Index of the Record in the records list
	 * @param row the selected row (may be same as recIndex, not sure)
	 */
	public Object getRecord(int recIndex) {
	
		if (recIndex >= 0 && recIndex < recordList.size()) {
		    Object tempRec =  recordList.get(recIndex);
		    return tempRec;
		}
		
		return null;
	}

	/** Returns a list of all records contained in this table */
	public ArrayList getRecordList() {
		return new ArrayList(recordList);
	}

	/** Returns the inded of the specified Record in the records list */
	public int indexOf(String rec) {
		return recordList.indexOf(rec);
	}
	
	boolean existsPattern (String chkPatt) {
	    
	    String cmpRec = chkPatt.toLowerCase();
	    ListIterator itr = recordList.listIterator();
	    while (itr.hasNext()) {
	        OutboundRecord currRec = (OutboundRecord) itr.next();
	        if (currRec.pattern.toLowerCase().equals (cmpRec)) {
	            return true;
	        }
	    }
	    return false;
	}

	String conflictsRecord (String patt, boolean editFlag, OutboundRecord myRec) {
	    
	    String retStr = "";
	   
	    String tmpEndPointName = null;
	    if (myRec != null) {
	        tmpEndPointName = myRec.getFirstHop().endPointName;
	    }
	    
	    String newPattern = patt.toLowerCase();
	    Iterator itr = recordList.iterator();
	    while (itr.hasNext()) {
	        OutboundRecord currRec = (OutboundRecord) itr.next();
		    
	        String currPattern = currRec.pattern.toLowerCase();
	        if (currPattern.startsWith("*")) {
	            String tmpSuffix = currPattern.substring(1);
	            if (newPattern.endsWith(tmpSuffix)) {
	                
	                if (editFlag && tmpEndPointName != null && currRec.getFirstHop().endPointName.equals(tmpEndPointName) && currRec.getFirstHop().ocdId.equals (SecureConnectionWizardDialog.RSP_TUNNEL_OCD_ID)) {
	                    // do nothing
	                } else {
	                    retStr += currPattern + " ";
	                }
	            }
	        } else {
	            if (currPattern.equals (newPattern)) {
	                if (editFlag &&  tmpEndPointName != null && currRec.getFirstHop().endPointName.equals(tmpEndPointName) && currRec.getFirstHop().ocdId.equals (SecureConnectionWizardDialog.RSP_TUNNEL_OCD_ID)) {
	                    // do nothing
	                } else {
	                    retStr += currPattern + " ";
	                }
	            }
	        }
	    }
	    return retStr;	        	
	}
	
	/** Returns a string representation of all the URLs/URL patterns in this
	 * table.  
	 * @return NULL if no URLs/URL patterns
	 */
	String getURLs () {
	    
	    if (recordList.isEmpty()) {
	        return null;
	    } else {
	        
	        String retStr = "";
	        Iterator itr = recordList.iterator();
	        while (itr.hasNext()) {
	            OutboundRecord currRec = (OutboundRecord) itr.next();
	            retStr += currRec.pattern;
	            if (itr.hasNext()) {
	                retStr += ", ";
	            }
	        }
	        return retStr;	    
	    }
	}
}
